SystemOrganization addCategory: #'OmniBrowser-Tests'!
SystemOrganization addCategory: #'OmniBrowser-Kernel'!
SystemOrganization addCategory: #'OmniBrowser-Morphs'!
SystemOrganization addCategory: #'OmniBrowser-Lenses'!

RectangleMorph subclass: #OBColumnScroller
	instanceVariableNames: 'background transform scrollBar '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphs'!

!OBColumnScroller methodsFor: 'scrolling' stamp: 'cwp 8/25/2003 21:14'!
setScrollDeltas
	| range interval value |
	transform hasSubmorphs ifFalse: [scrollBar interval: 1.0. ^ self].
	range _ self leftoverScrollRange.
	range = 0 ifTrue: [^ scrollBar interval: 1.0; setValue: 0].
	interval _ ((self innerBounds width) / self totalScrollRange) asFloat.
	value _ (transform offset x / range min: 1.0) asFloat.
	scrollBar interval: interval.
	scrollBar setValue: value.! !

!OBColumnScroller methodsFor: 'columns' stamp: 'cwp 8/24/2003 14:30'!
addColumn: aMorph
	aMorph 
		borderWidth: 0;
		hResizing: #rigid;
		vResizing: #rigid;
		layoutInset: 0.
	transform hasSubmorphs ifTrue: [transform addMorphBack: self separator].
	transform addMorphBack: aMorph! !

!OBColumnScroller methodsFor: 'layout' stamp: 'cwp 8/25/2003 22:13'!
layoutWidgets
	| inner outer |
	outer _ super innerBounds.
	inner _ self innerBounds.
	transform bounds: inner.
	scrollBar bounds: ((inner left @ inner bottom + 1)
						corner: outer bottomRight)! !

!OBColumnScroller methodsFor: 'layout' stamp: 'cwp 8/25/2003 22:02'!
innerBounds
	| rect |
	rect _ super innerBounds.
	^ rect withHeight: rect height - self scrollBarHeight - 1! !

!OBColumnScroller methodsFor: 'input events' stamp: 'cwp 8/25/2003 20:33'!
scrollBarValue: value
	transform hasSubmorphs ifFalse: [^ self].
	transform offset: (self leftoverScrollRange * value) rounded @ 0.! !

!OBColumnScroller methodsFor: 'layout' stamp: 'cwp 8/25/2003 20:28'!
columnWidth
	| c total |
	c _ self columnCount.
	total _ self innerBounds width.
	c = 0 ifTrue: [^ self minColumnWidth].
	^ self minColumnWidth max: (total - (c - 1 * self separatorWidth) / c) floor! !

!OBColumnScroller methodsFor: 'defaults' stamp: 'cwp 8/24/2003 15:13'!
separatorWidth
	^ 4! !

!OBColumnScroller methodsFor: 'layout' stamp: 'cwp 8/25/2003 19:50'!
columnCount
	^ (transform submorphs size / 2) ceiling.! !

!OBColumnScroller methodsFor: 'initialization' stamp: 'cwp 8/25/2003 19:34'!
initializeTransform
	transform _ TransformMorph new.
	transform
		color: Color transparent;
		borderWidth: 0;
		vResizing: #spaceFill;
		hResizing: #spaceFill;
		disableTableLayout.
	self addMorphBack: transform.
! !

!OBColumnScroller methodsFor: 'layout' stamp: 'cwp 8/25/2003 21:20'!
bounds: aRectangle
	super bounds: aRectangle.
	self doLayout.! !

!OBColumnScroller methodsFor: 'scrolling' stamp: 'cwp 8/23/2003 16:21'!
scrollDeltaWidth
	^ 1! !

!OBColumnScroller methodsFor: 'columns' stamp: 'cwp 11/23/2003 23:30'!
clearColumns
	transform removeAllMorphs! !

!OBColumnScroller methodsFor: 'initialization' stamp: 'cwp 11/23/2003 18:07'!
initializeScrollbar
	scrollBar _ OBScrollBar new model: self slotName: 'scrollBar'.
	scrollBar 
		borderWidth: 0;
		borderColor: #inset;
		height: self scrollBarHeight.
	self addMorphBack: scrollBar.! !

!OBColumnScroller methodsFor: 'initialization' stamp: 'cwp 8/25/2003 21:24'!
initialize
	super initialize.
	self 
		color: Color red;
		borderWidth: 0;
		vResizing: #spaceFill;
		hResizing: #spaceFill.
	self 
		initializeTransform;
		initializeScrollbar.
! !

!OBColumnScroller methodsFor: 'columns' stamp: 'cwp 8/24/2003 15:15'!
separator
	^ BorderedSubpaneDividerMorph vertical 
		borderWidth: self separatorWidth / 2;
		color: Color lightGray;
		borderRaised.! !

!OBColumnScroller methodsFor: 'input events' stamp: 'cwp 8/22/2003 19:17'!
hideOrShowScrollBar
	! !

!OBColumnScroller methodsFor: 'layout' stamp: 'cwp 11/23/2003 22:43'!
extraPixelsForColumnWidth: width
	| c extra |
	c _ self columnCount.
	c = 0 ifTrue: [^ 0].
	extra _ self innerBounds width - (width * c) - (c - 1 * self separatorWidth).
	^ extra > 0 ifTrue: [extra] ifFalse: [0]	! !

!OBColumnScroller methodsFor: 'layout' stamp: 'cwp 8/24/2003 14:29'!
scrollBarHeight
	^ 12! !

!OBColumnScroller methodsFor: 'layout' stamp: 'cwp 8/25/2003 20:15'!
computeMorphWidths
	| morphs widths columnWidth extra |
	morphs _ transform submorphs.
	widths _ Array new: morphs size.
	columnWidth _ self columnWidth.
	extra _ self extraPixelsForColumnWidth: columnWidth.
	(2 to: morphs size by: 2) do: [:i | widths at: i put: self separatorWidth].
	(1 to: extra by: 2) do: [:i | widths at: i put: columnWidth + 1].
	extra _ 1 max: (extra roundTo: 2) - 1.
	(extra to: morphs size by: 2) do: [:i | widths at: i put: columnWidth].
	^ widths! !

!OBColumnScroller methodsFor: 'layout' stamp: 'cwp 8/24/2003 14:55'!
columnHeight
	^ transform bounds height! !

!OBColumnScroller methodsFor: 'scrolling' stamp: 'cwp 8/25/2003 21:04'!
leftoverScrollRange
	^ (self totalScrollRange - self innerBounds width roundTo: self scrollDeltaWidth) max: 0
! !

!OBColumnScroller methodsFor: 'scrolling' stamp: 'cwp 8/25/2003 19:27'!
totalScrollRange
	| submorphBounds |
	submorphBounds := transform localSubmorphBounds ifNil: [^ 0].
	^ submorphBounds width
! !

!OBColumnScroller methodsFor: 'layout' stamp: 'cwp 8/24/2003 17:33'!
layoutColumns
	| widths rect |
	widths _ self computeMorphWidths.
	rect _ 0@0 extent: (0 @ self columnHeight).
	transform submorphs 
					with: widths 
					do: [:m :w | 
						rect _ rect withWidth: w.
						m bounds: rect.
						rect _ rect translateBy: (w@0)]
						
! !

!OBColumnScroller methodsFor: 'defaults' stamp: 'cwp 8/25/2003 20:28'!
minColumnWidth
	^ 120! !

!OBColumnScroller methodsFor: 'layout' stamp: 'cwp 8/25/2003 21:29'!
doLayout
	self layoutWidgets.
	self layoutColumns.
	self setScrollDeltas! !

Slider subclass: #OBScrollBar
	instanceVariableNames: 'upButton downButton pagingArea scrollDelta pageDelta interval menuSelector timeOfMouseDown timeOfLastScroll nextPageDirection currentScrollDelay '
	classVariableNames: 'CachedImages UpArrow8Bit UpArrow '
	poolDictionaries: ''
	category: 'OmniBrowser-Morphs'!

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
extent: p
        p x > p y
                ifTrue: [super extent: (p max: 42@8)]
                ifFalse: [super extent: (p max: 8@42)]! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:07'!
totalSliderArea
	| upperBoundsButton |
	upperBoundsButton _ upButton.
	upButton bottom > upperBoundsButton bottom
		ifTrue: [upperBoundsButton _ upButton].
	^ bounds isWide
		ifTrue: [upperBoundsButton bounds topRight corner: downButton bounds bottomLeft]
		ifFalse: [upperBoundsButton bounds bottomLeft corner: downButton bounds topRight].
! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
alternativeScrollbarLook
	^Preferences alternativeScrollbarLook! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollDown
	self flag: #obsolete.
	downButton eventHandler: nil.
	downButton on: #mouseDown send: #scrollDownInit to: self.
	downButton on: #mouseUp send: #finishedScrolling to: self.
	^self scrollDownInit! !

!OBScrollBar methodsFor: 'scroll timing' stamp: 'cwp 11/23/2003 18:07'!
resetTimer
	timeOfMouseDown _ Time millisecondClockValue.
	timeOfLastScroll _ timeOfMouseDown - 1000 max: 0.
	nextPageDirection _ nil.
	currentScrollDelay _ nil! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
scrollDelta
	^ scrollDelta! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollUpInit
	upButton borderInset.
	self resetTimer.
	self scrollBarAction: #doScrollUp.
	self startStepping.! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:07'!
sliderThickness
	^ self roundedScrollbarLook ifTrue:[15] ifFalse:[super sliderThickness]! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializePagingArea
	pagingArea := RectangleMorph newBounds: self totalSliderArea
								color: (Color r: 0.6 g: 0.6 b: 0.8).
	pagingArea borderWidth: 0.
	pagingArea on: #mouseDown send: #scrollPageInit: to: self.
	pagingArea on: #mouseUp send: #finishedScrolling to: self.
	self addMorph: pagingArea.
	self roundedScrollbarLook 
		ifTrue:[pagingArea color: (Color gray: 0.9)].! !

!OBScrollBar methodsFor: 'testing' stamp: 'cwp 11/23/2003 18:06'!
stepTime
	^ currentScrollDelay ifNil: [300]! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollBarAction
	^self valueOfProperty: #scrollBarAction! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeDownButton
	downButton := RectangleMorph
		newBounds: (self innerBounds bottomRight - self buttonExtent extent: self buttonExtent)
		color: self thumbColor.
	downButton on: #mouseDown send: #scrollDownInit to: self.
	downButton on: #mouseUp send: #finishedScrolling to: self.
	downButton addMorphCentered: (ImageMorph new image: 
		(self 
			cachedImageAt: (bounds isWide ifTrue: ['right'] ifFalse: ['down']) 
			ifAbsentPut: [
				self upArrow8Bit
					rotateBy: (bounds isWide ifTrue: [#right] ifFalse: [#pi])
					centerAt: 0@0
			]
		)
	).
	self roundedScrollbarLook ifTrue:[
		downButton color: Color veryLightGray.
		downButton borderStyle: (BorderStyle complexRaised width: 3).
	] ifFalse:[
		downButton setBorderWidth: 1 borderColor: #raised.
	].
	self addMorph: downButton.
! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
setNextDirectionFromEvent: event

	nextPageDirection _ bounds isWide ifTrue: [
		event cursorPoint x >= slider center x
	]
	ifFalse: [
		event cursorPoint y >= slider center y
	]

! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollDownInit
	downButton borderInset.
	self resetTimer.
	self scrollBarAction: #doScrollDown.
	self startStepping.! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeEmbedded: aBool
	"aBool == true => inboard scrollbar
	aBool == false => flop-out scrollbar"
	self roundedScrollbarLook ifFalse:[^self].
	aBool ifTrue:[
		self borderStyle: (BorderStyle inset width: 2).
		self cornerStyle: #square.
	] ifFalse:[
		self borderStyle: (BorderStyle width: 1 color: Color black).
		self cornerStyle: #rounded.
	].
	self removeAllMorphs.
	self initializeSlider.! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
sliderShadowColor
	^ self roundedScrollbarLook
		ifTrue: [self sliderColor darker]
		ifFalse: [super sliderShadowColor]
! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollPageInit: evt
	self resetTimer.
	self setNextDirectionFromEvent: evt.
	self scrollBarAction: #doScrollByPage.
	self startStepping.! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
interval: d
	"Supply an optional floating fraction so slider can expand to indicate range"
	interval _ d min: 1.0.
	self expandSlider.
	self computeSlider.! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollUp: count
	self setValue: (value - (scrollDelta * count) - 0.000001 max: 0.0)! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
sliderExtent
	"The sliderExtent is now stored in the slider itself, not hardcoded as it is in the superclass."
	^slider extent! !

!OBScrollBar methodsFor: 'initialization' stamp: 'cwp 11/23/2003 18:07'!
initialize
	super initialize.
	scrollDelta _ 0.02.
	pageDelta _ 0.2.
	self roundedScrollbarLook ifTrue:[
		self borderStyle: ((BorderStyle complexFramed width: 2) "baseColor: Color gray")].! !

!OBScrollBar methodsFor: 'testing' stamp: 'cwp 11/23/2003 18:06'!
wantsSteps
	^self scrollBarAction notNil! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollUp
	self flag: #obsolete.
	upButton eventHandler: nil.
	upButton on: #mouseDown send: #scrollUpInit to: self.
	upButton on: #mouseUp send: #finishedScrolling to: self.
	^self scrollUpInit! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
doScrollDown
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 200 delay2: 40) ifFalse: [^ self].
	self setValue: (value + scrollDelta + 0.000001 min: 1.0)! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
doScrollByPage
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 300 delay2: 100) ifFalse: [^ self].
	nextPageDirection
		ifTrue: [self setValue: (value + pageDelta min: 1.0)]
		ifFalse: [self setValue: (value - pageDelta max: 0.0)]
! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
finishedScrolling
	self stopStepping.
	self scrollBarAction: nil.
	self roundedScrollbarLook ifTrue:[
		upButton borderStyle: (BorderStyle complexRaised width: upButton borderWidth).
		downButton borderStyle: (BorderStyle complexRaised width: downButton borderWidth).
	] ifFalse:[
		downButton borderRaised.
		upButton borderRaised.
	].

! !

!OBScrollBar class methodsFor: 'as yet unclassified' stamp: 'cwp 11/23/2003 18:07'!
alwaysShowFlatScrollbarForAlternativeLook
	"Set this value to true, if you want to see the flat scrollbar look in flop-out mode as well as inboard.  Otherwise the flop-out scrollbar will be rounded and inboard will be flat."
	^ false! !

!OBScrollBar methodsFor: 'other events' stamp: 'cwp 11/23/2003 18:06'!
mouseDownInSlider: event
	interval = 1.0 ifTrue:
		["make the entire scrollable area visible if a full scrollbar is clicked on"
		self setValue: 0.
		self model hideOrShowScrollBar].
	super mouseDownInSlider: event! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeSlider
	self roundedScrollbarLook ifTrue:[
		self 
			initializeUpButton;
			initializeDownButton;
			initializePagingArea.
	] ifFalse:[
		self initializeUpButton; 
			initializeDownButton; 
			initializePagingArea.
	].
	super initializeSlider.
	self roundedScrollbarLook ifTrue:[
		slider cornerStyle: #rounded.
		slider borderStyle: (BorderStyle complexRaised width: 3).
		sliderShadow cornerStyle: #rounded.
	].
	self sliderColor: self sliderColor.! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
scrollDelta: d1 pageDelta: d2
	"Supply optional increments for better scrolling of, eg, text"
	scrollDelta _ d1.
	pageDelta _ d2.! !

!OBScrollBar methodsFor: 'accessing' stamp: 'cwp 11/23/2003 18:06'!
adoptPaneColor: aColor
	"Adopt the given pane color"
	self alternativeScrollbarLook ifFalse:[^self].
	aColor ifNil:[^self].
	self sliderColor: aColor.! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollDown: count
	self setValue: (value + (scrollDelta * count) + 0.000001 min: 1.0)! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollBarAction: aSymbol
	self setProperty: #scrollBarAction toValue: aSymbol! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeUpButton
	upButton := self roundedScrollbarLook 
				ifTrue:  
					[RectangleMorph 
						newBounds: (self innerBounds topLeft extent: self buttonExtent)]
				ifFalse: 
					[RectangleMorph 
						newBounds: ((self innerBounds topLeft) extent: self buttonExtent)].
	upButton color: self thumbColor.
	upButton 
		on: #mouseDown
		send: #scrollUpInit
		to: self.
	upButton 
		on: #mouseUp
		send: #finishedScrolling
		to: self.
	upButton 
		addMorphCentered: (ImageMorph new image: (self 
						cachedImageAt: (bounds isWide ifTrue: ['left'] ifFalse: ['up'])
						ifAbsentPut: 
							[bounds isWide 
								ifTrue: [self upArrow8Bit rotateBy: #left centerAt: 0 @ 0]
								ifFalse: [self upArrow8Bit]])).
	self roundedScrollbarLook 
		ifTrue: 
			[upButton color: Color veryLightGray.
			upButton borderStyle: (BorderStyle complexRaised width: 3)]
		ifFalse: [upButton setBorderWidth: 1 borderColor: #raised].
	self addMorph: upButton! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
expandSlider
	"Compute the new size of the slider (use the old sliderThickness as a minimum)."
	| r |
	r _ self totalSliderArea.
	slider extent: (bounds isWide
		ifTrue: [((r width * interval) asInteger max: self sliderThickness) @ slider height]
		ifFalse: [slider width @ ((r height * interval) asInteger max: self sliderThickness)])! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
doScrollUp
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 200 delay2: 40) ifFalse: [^ self].
	self setValue: (value - scrollDelta - 0.000001 max: 0.0)! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
thumbColor
	"Problem: Part of the ScrollBar/Slider code uses 'slider' to mean the entire scrollbar/slider widget, and part of it uses 'slider' to mean only the draggable 'thumb'.  This should be cleaned up so that 'thumb' is used instead of 'slider' where appropriate.  For now, the meaning of thumbColor is clear, at least."

	^ self alternativeScrollbarLook 
		ifTrue: [self sliderColor alphaMixed: 0.7 with: (Color gray: 0.95)]
		ifFalse: [Color veryLightGray]! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
pagingArea
	^pagingArea! !

!OBScrollBar methodsFor: 'stepping and presenter' stamp: 'cwp 11/23/2003 18:07'!
step
	| action |
	action _ self scrollBarAction.
	action ifNotNil:[self perform: action].! !

!OBScrollBar methodsFor: 'scroll timing' stamp: 'cwp 11/23/2003 18:07'!
waitForDelay1: delay1 delay2: delay2 
	"Return true if an appropriate delay has passed since the last scroll operation.
	The delay decreases exponentially from delay1 to delay2."

	| now scrollDelay |
	timeOfLastScroll isNil ifTrue: [self resetTimer].	"Only needed for old instances"
	now := Time millisecondClockValue.
	(scrollDelay := currentScrollDelay) isNil 
		ifTrue: [scrollDelay := delay1	"initial delay"].
	currentScrollDelay := scrollDelay * 9 // 10 max: delay2.	"decrease the delay"
	timeOfLastScroll := now.
	^true! !

!OBScrollBar methodsFor: 'as yet unclassified' stamp: 'cwp 11/23/2003 18:06'!
cachedImageAt: aKey ifAbsentPut: aBlock

	CachedImages ifNil: [CachedImages _ Dictionary new].
	^CachedImages at: aKey ifAbsentPut: aBlock! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
roundedScrollbarLook
	"Rounded look currently only shows up in flop-out mode"
	^Preferences alternativeScrollbarLook and: [Preferences inboardScrollbars not and: [self class alwaysShowFlatScrollbarForAlternativeLook not]]
! !

!OBScrollBar methodsFor: 'other events' stamp: 'cwp 11/23/2003 18:06'!
menuButtonMouseDown: event
	event hand showTemporaryCursor: nil.
	self use: menuSelector orMakeModelSelectorFor: 'MenuButtonPressed:'
		in: [:sel | menuSelector _ sel.  model perform: sel with: event]! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
buttonExtent
	^ bounds isWide
		ifTrue: [11 @ self innerBounds height]
		ifFalse: [self innerBounds width @ 11]! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
upArrow8Bit

	"convert to 8-bit and convert white to transparent to avoid gratuitous conversion every time we put one in an ImageMorph"

	^UpArrow8Bit ifNil: [
		UpArrow8Bit _ (ColorForm mappingWhiteToTransparentFrom: UpArrow) asFormOfDepth: 8
	]! !

!OBScrollBar class methodsFor: 'class initialization' stamp: 'cwp 11/23/2003 18:07'!
initialize "ScrollBar initialize"

	UpArrow _ Form
		extent: 6@3
		fromArray: #(2r11e28 2r1111e27 2r111111e26)
		offset: 0@0.! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
sliderColor: aColor
	"Change the color of the scrollbar to go with aColor."
	| buttonColor |
	super sliderColor: aColor.
	buttonColor _ self thumbColor.
	upButton color: buttonColor.
	downButton color: buttonColor.
	slider color: buttonColor.
	self alternativeScrollbarLook ifTrue:
		[self roundedScrollbarLook
			ifTrue:
				[self color: Color transparent.
				pagingArea color: aColor muchLighter.
				self borderStyle style == #simple 
					ifTrue:[self borderColor: aColor darker darker]
					ifFalse:[self borderStyle baseColor: aColor]]
			ifFalse:
				[pagingArea color: (aColor alphaMixed: 0.3 with: Color white).
				self borderWidth: 0]]
! !

!OBScrollBar methodsFor: 'model access' stamp: 'cwp 11/23/2003 18:06'!
setValue: newValue
	"Using roundTo: instead of truncateTo: ensures that scrollUp will scroll the same distance as scrollDown."
	^ super setValue: (newValue roundTo: scrollDelta)! !

TestCase subclass: #OBDirectoryLensTest
	instanceVariableNames: 'lens '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBDirectoryLensTest methodsFor: 'running' stamp: 'cwp 11/28/2003 22:43'!
setUp
	lens _ OBDirectoryLens on: self dataDir.! !

!OBDirectoryLensTest methodsFor: 'testing' stamp: 'cwp 11/28/2003 22:51'!
testItems
	self assert: lens items = #(alpha beta delta gamma).! !

!OBDirectoryLensTest methodsFor: 'testing' stamp: 'cwp 11/28/2003 23:01'!
testChildren
	| children |
	children _ lens lensesFor: #alpha.
	self assert: children size = 0.
	children _ lens lensesFor: #delta.
	self assert: children size = 1.
	self assert: children first items = #(epsilon).! !

!OBDirectoryLensTest methodsFor: 'data' stamp: 'cwp 11/28/2003 22:47'!
dataDir
	^ OBDirectoryResource current dataDir! !

TestCase subclass: #OBBrowserTest
	instanceVariableNames: 'model morph '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBBrowserTest methodsFor: 'testing' stamp: 'cwp 11/29/2003 17:36'!
testPanes
	self assert: (self morphsOfClass: PluggableListMorph) size = 1.
	self assert: (self morphsOfClass: PluggableTextMorph) size = 1.! !

!OBBrowserTest methodsFor: 'testing' stamp: 'cwp 11/23/2003 13:12'!
testCreateWindow
	self assert: (morph isKindOf: SystemWindow).
	self assert: (morph label = 'OmniBrowser').
! !

!OBBrowserTest methodsFor: 'testing' stamp: 'cwp 12/2/2003 21:31'!
testSelectC
	self select: #a.
	self assertListAt: 1 contains: #(a b c).
	self assert: self listMorphs size = 1.! !

!OBBrowserTest methodsFor: 'running' stamp: 'cwp 11/23/2003 13:24'!
setUp
	model _ OBBrowser onLens: self fakeLens.
	morph _ model window.! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 11/7/2003 09:03'!
deselect: aString
	| listMorph |
	listMorph _ self findListContaining: aString.
	listMorph changeModelSelection: 0.! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 11/23/2003 13:38'!
listMorphs
	^ self morphsOfClass: PluggableListMorph! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 11/23/2003 13:39'!
assertAListIncludes: anArrayOfStrings
	self listMorphs 
			detect: [:m | m getList includesAllOf: anArrayOfStrings]
			ifNone: [self assert: false].! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 11/23/2003 13:37'!
morphsOfClass: aClass
	| found |
	found _ OrderedCollection new.
	morph allMorphsDo: [:m | (m isMemberOf: aClass) ifTrue: [found add: m]].
	^ found! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 12/2/2003 21:37'!
assertTextIs: aString
	self assert: self textMorph text = aString.! !

!OBBrowserTest methodsFor: 'testing' stamp: 'cwp 12/2/2003 21:34'!
testInitialDisplay
	| lists |
	lists _ self listMorphs.
	self assert: lists size = 1.
	self assert: (lists first getList includesAllOf: #(a b c)).
	self assertTextIs: ''.
	! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 11/23/2003 14:46'!
findListContaining: aString
	^ self listMorphs detect: [:m | m getList includes: aString] ifNone: []! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 11/23/2003 14:47'!
select: aString
	| listMorph |
	listMorph _ self findListContaining: aString.
	self assert: listMorph notNil.
	listMorph changeModelSelection: (listMorph getList indexOf: aString).! !

!OBBrowserTest methodsFor: 'testing' stamp: 'cwp 12/2/2003 21:30'!
testSelectA
	self select: #a.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #(aa ab ac).! !

!OBBrowserTest methodsFor: 'testing' stamp: 'cwp 12/2/2003 21:30'!
testDeselectA
	self select: #a.
	self deselect: #a.
	self assertListAt: 1 contains: #(a b c).
	self assert: self listMorphs size = 1.! !

!OBBrowserTest methodsFor: 'running' stamp: 'cwp 11/23/2003 13:05'!
fakeLens
	^ OBFakeLens new! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 11/7/2003 08:43'!
assertListAt: index contains: items
	| list |
	list _ (self listMorphs at: index) getList.
	self assert: (list size = items size).
	self assert: (list includesAllOf: items).! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 12/2/2003 21:36'!
textMorph
	^ (self morphsOfClass: PluggableTextMorph) last! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 11/23/2003 13:39'!
assertAListMatches: strings
	| listMorphs list |
	listMorphs _ self listMorphs.
	listMorphs 
		detect: [:m | list _ m getList. (list size = strings size) and: [list includesAllOf: strings]]
		ifNone: [self assert: false].! !

TestCase subclass: #OBColumnScrollerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBColumnScrollerTest methodsFor: 'as yet unclassified' stamp: 'cwp 11/23/2003 22:33'!
testExtraPixels
	| scroller |
	(scroller _ OBColumnScroller new)
		addColumn: self createColumn morph;
		addColumn: self createColumn morph.
	self assert: (scroller extraPixelsForColumnWidth: scroller columnWidth) < 2
	! !

!OBColumnScrollerTest methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2003 21:40'!
createColumn
	| lens |
	lens _ OBFakeLens new.
	^ OBColumn inBrowser: (OBBrowser onLens: lens) withLens: lens.! !

TestResource subclass: #OBDirectoryResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBDirectoryResource methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2003 22:46'!
tearDown
	self dataDir recursiveDelete! !

!OBDirectoryResource methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2003 22:44'!
dataDirName
	^ 'obtest'! !

!OBDirectoryResource methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2003 22:47'!
dataDir
	^ FileDirectory default directoryNamed: self dataDirName! !

!OBDirectoryResource methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2003 22:49'!
setUp
	| dir |
	dir _ self dataDir.
	dir assureExistence.
	#(alpha beta gamma) do: [:filename | | file | file _ dir forceNewFileNamed: filename.
							  file nextPutAll: filename.
							  file close].
	dir createDirectory: 'delta'.
	(dir directoryNamed: 'delta') createDirectory: 'epsilon'.! !

Object subclass: #OBBrowser
	instanceVariableNames: 'scroller firstColumn currentColumn '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBBrowser methodsFor: 'updating' stamp: 'cwp 11/29/2003 17:40'!
currentColumn: aColumn
	currentColumn _ aColumn.
	scroller clearColumns.
	firstColumn do: [:c | scroller addColumn: c morph].
	scroller doLayout.
	self changed: #text.! !

!OBBrowser methodsFor: 'constructing' stamp: 'cwp 11/23/2003 18:02'!
label
	^ 'OmniBrowser'! !

!OBBrowser methodsFor: 'constructing' stamp: 'cwp 11/29/2003 17:37'!
window
	| window |
	window _ SystemWindow labelled: self label.
	window model: self.
	window 
		addMorph: self scroller
		fullFrame: (LayoutFrame fractions: (0@0 extent: 1@0.4)).
	window
		addMorph: self textPane
		fullFrame: (LayoutFrame fractions: (0@0.4 corner: 1@1)).
	^ window! !

!OBBrowser methodsFor: 'callbacks' stamp: 'cwp 11/29/2003 17:39'!
text: aText
! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 11/29/2003 17:46'!
currentLens
	^ currentColumn selectedLens! !

!OBBrowser methodsFor: 'initializing' stamp: 'cwp 11/29/2003 17:41'!
setRootLens: aLens
	firstColumn _ OBColumn inBrowser: self withLens: aLens.
	currentColumn _ firstColumn! !

!OBBrowser methodsFor: 'callbacks' stamp: 'cwp 11/29/2003 17:45'!
text
	^ self currentLens textFor: currentColumn selectedItem.! !

!OBBrowser class methodsFor: 'as yet unclassified' stamp: 'cwp 11/5/2003 19:41'!
onLens: aLens
	^ self new setRootLens: aLens! !

!OBBrowser methodsFor: 'constructing' stamp: 'cwp 11/29/2003 17:38'!
textPane
	^ PluggableTextMorph
		on: self
		text: #text
		accept: #text:! !

!OBBrowser methodsFor: 'constructing' stamp: 'cwp 11/23/2003 23:28'!
scroller
	^ scroller ifNil: [scroller _ OBColumnScroller new.
					scroller 
						vResizing: #spaceFill;
						hResizing: #spaceFill.
					firstColumn do: [:col | scroller addColumn: col morph].
					scroller]
	! !

Object subclass: #OBLens
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

OBLens subclass: #OBFakeLens
	instanceVariableNames: 'items '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBFakeLens methodsFor: 'as yet unclassified' stamp: 'cwp 12/2/2003 21:32'!
lensesFor: anItem
	| children |
	^ anItem last = $c
		ifTrue: [Array new]
		ifFalse: [children _ #($a $b $c) collect: [:i | anItem, i asString].
				Array with: (self class new items: children)]! !

!OBFakeLens methodsFor: 'as yet unclassified' stamp: 'cwp 11/25/2003 21:17'!
items
	^ items ifNil: [#(a b c)]! !

!OBFakeLens methodsFor: 'as yet unclassified' stamp: 'cwp 11/23/2003 13:13'!
items: anArray
	items _ anArray! !

!OBLens methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2003 23:06'!
items
	^ #()! !

!OBLens methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2003 23:07'!
lensesFor: anItem
	^ #()! !

!OBLens methodsFor: 'as yet unclassified' stamp: 'cwp 12/2/2003 21:38'!
textFor: anItem
	^ anItem 
		ifNil: [String new]
		ifNotNil: [anItem asString]! !

OBLens subclass: #OBDirectoryLens
	instanceVariableNames: 'directory '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Lenses'!

!OBDirectoryLens methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2003 21:57'!
directory: aFileDirectory
	directory _ aFileDirectory! !

!OBDirectoryLens class methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2003 21:57'!
on: aFileDirectory
	^ self new directory: aFileDirectory! !

!OBDirectoryLens methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2003 22:01'!
items
	^ directory fileAndDirectoryNames! !

!OBDirectoryLens methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2003 23:01'!
lensesFor: anItem
	| entry |
	entry _ directory entryAt: anItem.
	^ (entry at: 4) 
		ifTrue: [Array with: (self class on: (directory directoryNamed: anItem))]
		ifFalse: [#()].
	! !

Object subclass: #OBColumn
	instanceVariableNames: 'browser next lenses selection '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/5/2003 19:43'!
lens: aLens
	self lenses: (Array with: aLens)! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 11/7/2003 08:50'!
selectedLens
	^ lenses first! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 11/7/2003 08:48'!
selection: anInteger
	selection _ anInteger.
	self updateNext.
	self changed: #selection.! !

!OBColumn methodsFor: 'enumerating' stamp: 'cwp 11/23/2003 14:59'!
do: aBlock
	aBlock value: self.
	next ifNotNil: [next do: aBlock]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 11/5/2003 19:01'!
selection
	^ selection ifNil: [0]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 11/23/2003 13:14'!
list
	^ lenses 
		ifNil: [#()]
		ifNotNil: [lenses gather: [:ea | ea items]]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 11/28/2003 23:01'!
updateNext
	| lens |
	selection = 0 
		ifTrue: [self truncate]
		ifFalse: [self extend.
				lens _ self selectedLens.
				next lenses: (lens lensesFor: self selectedItem).
				next updateNext].
	browser currentColumn: self.! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/23/2003 18:59'!
morph
	^ PluggableListMorph
		on: self
		list: #list
		selected: #selection
		changeSelected: #selection:! !

!OBColumn class methodsFor: 'as yet unclassified' stamp: 'cwp 11/25/2003 21:00'!
inBrowser: aBrowser withLenses: aCollection
	^ self new 
			browser: aBrowser; 
			lenses: aCollection! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/25/2003 21:01'!
browser: aBrowser
	browser _ aBrowser! !

!OBColumn methodsFor: 'enumerating' stamp: 'cwp 11/23/2003 22:58'!
truncate
	next _ nil.! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/7/2003 08:56'!
lenses: aCollection
	lenses _ aCollection.
	selection _ 0.
	self changed: #list.! !

!OBColumn class methodsFor: 'as yet unclassified' stamp: 'cwp 11/25/2003 21:04'!
inBrowser: aBrowser 
	^ self inBrowser: aBrowser withLenses: Array new! !

!OBColumn methodsFor: 'enumerating' stamp: 'cwp 11/7/2003 08:20'!
next
	^ next! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 11/29/2003 17:43'!
selectedItem
	^ selection = 0 ifFalse: [self selectedLens items at: selection]
	! !

!OBColumn class methodsFor: 'as yet unclassified' stamp: 'cwp 11/25/2003 21:02'!
inBrowser: aBrowser withLens: aLens
	^ self inBrowser: aBrowser withLenses: (Array with: aLens)! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/23/2003 13:08'!
size: n
	"ensure that there are at least n-1 columns to the right of this one"

	n < 2 ifTrue: [^ self].
	next ifNil: [next _ self class new].
	next size: n - 1.! !

!OBColumn methodsFor: 'enumerating' stamp: 'cwp 11/25/2003 21:05'!
extend
	next ifNil: [next _ OBColumn inBrowser: browser]! !
OBScrollBar initialize!
